/***************************************************************************
 *   Copyright (C) 2007 by Lisandro Damián Nicanor Pérez Meyer             *
 *   perezmeyer@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GRAPHICSSCENE_H
#define GRAPHICSSCENE_H

#include <QGraphicsScene>
#include <QList>

// Forward declarations
class QGraphicsSceneMouseEvent;
class QGraphicsItem;
class QString;

class GraphicsScene : public QGraphicsScene
{
	Q_OBJECT

public:
	GraphicsScene(QObject * parent = 0);

public slots:
	/// Receives a signal saying if the Lagrange check box is selected or not
	void displayLagrangeCurve(int state);

	/// Receives a signal saying if the Hermite check box is selected or not
	void displayHermiteCurve(int state);

	/// Receives a signal saying if the de Casteljau check box is selected or not
	void displayDeCasteljauCurve(int state);

	/// Receives a signal saying if the Bernstein check box is selected or not
	void displayBernsteinCurve(int state);

	/// Receives a signal saying if the B-Spline check box is selected or not
	void displayBSplinesCurve(int state);

	/// Receives a signal saying if the cubic B-Spline check box is selected
	void displayCubicBSplineCurve(int state);

	/// B-Spline curve grade
	void bSplineCurveGradeChanged(int value);

signals:
	/// Sends a message to the status bar
	void sendSatusBarMessage(const QString & message, int timeout);

protected:
	/// Adds and removes items to/from the scene
	/**
		This function will add a point if a left click has been issued, or
		it will delete one if a point is found in the place where a right click
		happened.
	*/
	void mousePressEvent(QGraphicsSceneMouseEvent * mouseEvent);
// 	void mouseMoveEvent(QGraphicsSceneMouseEvent * mouseEvent);
// 	void mouseReleaseEvent(QGraphicsSceneMouseEvent * mouseEvent);

private:
	/// Generates the items needed to draw the curves
	void drawCurves();

	/// Draws a curve using the Lagrange algorythm
	void drawLagrange();

	/// Draws a curve using the Hermite algorythm
	void drawHermite();

	/// Draws a curve using the de Casteljau algorythm
	void drawDeCasteljau();

	/// Draws a curve using the Bernstein algorythm
	void drawBernstein();

	/// Draw a curve using the B-Splines algorythm
	void drawBSplines();

	/// Draw a curve using the cubic B-Spline algorythm
	void drawCubicBSpline();

	/// Returns the value at u of the spline base
	/**
	\param u curve local value
	\param i point to take into account
	\param grade grade of the sub curve
	*/
	double splineBase(double u, int i, int grade);

	/// Returns the value of ti
	double node(int i);

	/// List of points
	/**
		The scene does not store the points in the order they were created, so we
		need to do it ourselves.
	*/
	QList< QGraphicsItem * > pointsList;

	/// List of lines to generate the Lagrange curve
	QList < QGraphicsItem * > lagrangeLines;

	/// List of lines to generate the Hermite curve
	QList < QGraphicsItem * > hermiteLines;

	/// List of lines to generate the de Casteljau curve
	QList < QGraphicsItem * > deCasteljauLines;

	/// List of lines to generate the Bernstein curve
	QList < QGraphicsItem * > bernsteinLines;

	/// List of lines to generate the B-Splines curve
	QList < QGraphicsItem * > bSplineLines;

	/// List of lines to generate the cubic B-Spline curve
	QList < QGraphicsItem * > cubicBSplineLines;

	/// Draw the Lagrange curve
	bool lagrangeCurve;

	/// Draw the Hermite curve
	bool hermiteCurve;

	/// Draw the de Casteljau curve
	bool deCasteljauCurve;

	/// Draw the Bernstein curve
	bool bernsteinCurve;

	/// Draw the B-Splines curve
	bool bSplinesCurve;

	/// Draw the cubic B-Spline curve
	bool cubicBSplineCurve;

	/// Number of steps for the Lagrange algorythm
	static const int lagrangeStep = 100;

	/// Number of steps for the Hermite algorythm
	static const int hermiteStep = 30;

	/// Number of steps for the de Casteljau algorythm
	static const int deCasteljauStep = 100;

	/// Number of steps for the de Bernstein algorythm
	static const int bernsteinStep = 100;

	/// Number of steps for the B-Spline algorythm
	static const int bSplineStep = 100;

	/// Number of steps for the cubic B-Spline algorythm
	static const int cubicBSplineStep = 100;

	/// Grade of the B-Spline curve
	int bSplineGrade;

	/// List of points to form a B-Spline
	QList < QPointF> bSplinePoints;

	/// List of points to form a cubic B-Spline
	QList < QPointF > cubicBSplinePoints;
};

#endif // GraphicsScene



